/*
 * This simple example illustrates how to use the .NET engine client to 
 * interface with Apama. The example connects to a remote Event Correlator, 
 * creates an event consumer, creates some MonitorScript code and injects it, 
 * injects some sample events, and receives some back from the Correlator 
 * when the monitor triggers. It then disconnects from the Correlator and 
 * exits.
 *
 * $Copyright(c) 2010, 2012-2013 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013-2014 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 * $Version: 0 $
 */

using System;
using System.Threading;

using Apama;
using Apama.Event;
using Apama.Event.Parser;
using Apama.Engine;
using Apama.Engine.Client;
using Apama.Util;

public class EngineClientExample 
{
	private static readonly Logger logger = Logger.GetLogger(typeof(EngineClientExample));

	public static readonly Field<string> textField = FieldTypes.String.NewField("text");
	public static readonly EventType testEventType = new EventType("TestEvent", textField);
	public static readonly EventParser eventParser = new EventParser(testEventType);

	private void runSample(string host, int port)
	{
		// Create the engine client (does not connect until first method 
		// requiring remove connection is invoked)
		using (IEngineClient engineClient = EngineClientFactory.CreateEngineClient(host, port, "my-sample-process"))
		{
			// Listen for events sent to the "samplechannel" channel
			IConsumerOperations myConsumer = engineClient.AddConsumer("myConsumer", "samplechannel", "my-sample-process");
			myConsumer.EventReceived += OnConsumerEventReceived;

			// Optionally can monitor the connectedness of named consumers
			myConsumer.AddPropertyChangeListener(ConsumerPropertyChangeListener);

			// Inject some MonitorScript
			string epl = 
				"event TestEvent {" +"\n" +
				"	string text;" +"\n" +
				"}" +"\n" +
				"" +"\n" +
				"monitor Echo {" +"\n" +
				"" +"\n" +
				"	TestEvent test;" +"\n" +
				"" +"\n" +
				"	action onload {" +"\n" +
				"		on all TestEvent(*):test {" +"\n" +
				"			send TestEvent(test.text) to \"samplechannel\";" +"\n" +
				"		}" +"\n" +
				"	}" +"\n" +
				"}";
			engineClient.InjectMonitorScript(epl);

			// Wait a few seconds to be sure the injection event has been processed
			Thread.CurrentThread.Join(3000);

			// Get and display status statistics
			EngineStatus status = engineClient.GetRemoteStatus();
			Console.WriteLine(status);

			engineClient.SendEvents(
				new Event(testEventType).SetField(textField, "Hello, World"), 
				new Event(testEventType).SetField(textField, "Welcome to Apama")
				);

			// Delete the event type and monitor we added
			engineClient.DeleteName("Echo", false);
			engineClient.DeleteName("TestEvent", false);

			// Wait a few seconds for the output event to be received and the deletions processed
			Thread.CurrentThread.Join(3000);

			// Display status again
			status = engineClient.GetRemoteStatus();
			Console.WriteLine();
			Console.WriteLine(status);
		}

	}

	/// <summary>
	/// Listener to events received on channels that the consumer subscribed to.
	/// </summary>
	void OnConsumerEventReceived(Event evt) {
		evt = eventParser.Parse(evt.Text);
		Console.WriteLine("Event listener received event with message: '"+evt.GetField(textField)+"'");
	}
	
	/// <summary> 
	/// Listener to the PropertyChange events generated by the consumer.
	/// </summary>
	void ConsumerPropertyChangeListener(Object sender, PropertyChangedEventArgs evt) {
		if (evt.PropertyName.Equals(EngineClientConstants.PropertyConsumerConnected)) {
			IConsumerOperations theConsumer = (IConsumerOperations)sender;

			bool connected = (System.Boolean)evt.NewValue;
			Console.WriteLine("Property changed for consumer \"" + theConsumer.Name 
				+ "\": now "+ (connected ? "connected" : "disconnected"));
		}
	}

	public static int Main(string[] args)
	{
		if ((args.Length != 2) || (int.Parse(args[1]) <= 0)) {
			// Bad command line given
			Console.WriteLine("Usage: EngineClientExample.exe <host> <port>");
			return 100;
		}

		string host = args[0];
	    int port = int.Parse(args[1]);

		try 
		{
			new EngineClientExample().runSample(host, port);
			return 0;
		}
		catch (System.Exception ex) {
			logger.Error("Sample failed: ", ex);
			return 101;
		}
	}
}