// $Copyright(c) 2002 - 2010, 2013 Progress Software Corporation. All Rights Reserved.$ 
// $Copyright (c) 2013-2014 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
// Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 

using System;
using System.Threading;

using Apama;
using Apama.Event;
using Apama.Engine;
using Apama.Engine.Client;

/// <summary>
/// Event receiver implementation.
/// </summary>
public class ReceiveConsumer : EventConsumer {

	/// <summary>
	/// Receive events from the Engine and log them to stdout.
	/// </summary>
	///<param name="events">The received events.</param>
	public override void SendEvents(Event[] events) {
		if (events!=null) {
			for (int i=0; i<events.Length; i++) {
				Console.WriteLine(events[i]);
			}
		}
	}

} // non-public class ReceiveConsumer

/// <summary>
///Main program.
///Return codes:
///  0 = Everything OK
///  1 = Couldn't connect to Engine
///  2 = Something else went wrong
/// </summary>
public class Sample {

	public static int Main(string[] args) {
		// Return code
		int rc = 2;

		// Error message to display if anything goes wrong.  Update this
		// appropriately before each operation that might break.
		string emsg = null;

		if ((args.Length == 2) && (int.Parse(args[1]) > 0)) {

			// retrieving host and port
			string host = args[0];
			ushort port = ushort.Parse(args[1]);


			EngineManagement engine = null;
			ReceiveConsumer consumer = null;
			EventSupplier supplier = null;
			Apama.Engine.Api.EngineInit();
			try {
				try {
					rc = 1;

					// Attempt to connect to a remote Engine
					emsg = "Failed to connect to engine";

					engine = Apama.Engine.Api.ConnectToEngine(host, port);

					// Create an event consumer
					// The returned EventSupplier is a handle to private 
					// resources allocated by the engine to deal with this 
					// connection
					emsg = "Event sink connection failed";
					consumer = new ReceiveConsumer();
					supplier = engine.ConnectEventConsumer(consumer, new string[] {"output"});

					// Inject some MonitorScript (don't forget to delete it when done)
					emsg = "MonitorScript injection failed";
					string script = 
						"event TestEvent {" +"\n" +
						"	string text;" +"\n" +
						"}" +"\n" +
						"" +"\n" +
						"monitor Echo {" +"\n" +
						"" +"\n" +
						"	TestEvent test;" +"\n" +
						"" +"\n" +
						"	action onload {" +"\n" +
						"		on all TestEvent(*):test {" +"\n" +
						"			send TestEvent(test.text) to \"output\";" +"\n" +
						"		}" +"\n" +
						"	}" +"\n" +
						"}";
					engine.InjectMonitorScript(script);

					// Wait a few seconds to be sure the injection event has been processed
					Thread.CurrentThread.Join(3000);

					// Get & display status (have to delete it when done)
					emsg = "Status gathering failed";
					EngineStatus status = engine.GetStatus();
					Console.WriteLine(status);

					// Send some events
					emsg = "Event sending failed";
					Event[] events = new Event[2];
					events[0] = new Event("TestEvent(\"Hello, World\")");
					events[1] = new Event("TestEvent(\"Welcome to Apama\")");
					engine.SendEvents(events);

					// Delete the event type and monitor we added
					emsg = "Name deletion failed";
					engine.DeleteName("Echo");
					engine.DeleteName("TestEvent");

					// Wait a few seconds for the output event to be received and the deletions processed
					Thread.CurrentThread.Join(3000);

					// Display status again
					emsg = "Status gathering failed";
					Console.WriteLine();
					status = engine.GetStatus();
					Console.WriteLine(status);

					// Disconnect and destroy the event consumer
					emsg = "Event sink disconnection failed";


				} catch (EngineException ex) {
					// Rethrow so exception printer can deal with it
					throw ex;
				}
				catch (System.Exception sx) {
					throw new EngineException("Caught non-engine exception in Main(): " + sx.Message, sx);
				}
				
			}
			catch (EngineException ex) 
			{
				Console.WriteLine("{0}: {1}", emsg, ex);
			} 
			finally 
			{
				// since these objects use unmanaged resources it is essential to dispose them
				// be sure to do so in the correct order
				if (supplier != null) supplier.Dispose();
				if (consumer != null) consumer.Dispose();
				if (engine != null) engine.Dispose();
				Apama.Engine.Api.EngineShutdown();
			}

			// If we got this far, everything succeeded!
			rc = 0;

		}
		else {
			// Bad command line given
			Console.WriteLine("Usage: LowLevelEngineManagementSample.exe <host> <port>");
		}

		// Done!
		return rc;
	} // Main()

} 
